@@ -1,7 +1,3 @@
-# FIXME: optionally construct pointer
-
-constructors$data.table <- new.env()
-
 #' Constructive options for class 'data.table'
 #'
 #' These options will be used on objects of class 'data.table'.
@@ -12,7 +8,7 @@ constructors$data.table <- new.env()
 #'   on the object to see in which order the methods will be tried.
 #' * `"list"` : Use `list()` and treat the class as a regular attribute.
 #'
-#' @param constructor String. Name of the function used to construct the environment, see Details section.
+#' @param constructor String. Name of the function used to construct the object, see Details section.
 #' @param selfref Boolean. Whether to include the `.internal.selfref` attribute. It's
 #'   probably not useful, hence the default, `waldo::compare()` is used to assess the output
 #'   fidelity and doesn't check it, but if you really need to generate code that builds
@@ -21,30 +17,36 @@ constructors$data.table <- new.env()
 #' @return An object of class <constructive_options/constructive_options_data.table>
 #' @export
 opts_data.table <- function(constructor = c("data.table", "next", "list"), ..., selfref = FALSE) {
-  .cstr_combine_errors(
-    constructor <- .cstr_match_constructor(constructor, "data.table"),
-    check_dots_empty()
-  )
-  .cstr_options("data.table", constructor = constructor, selfref = selfref)
+  .cstr_options("data.table", constructor = constructor[[1]], ..., selfref = selfref)
 }
 
 #' @export
+#' @method .cstr_construct data.table
 .cstr_construct.data.table <- function(x, ...) {
-  opts <- .cstr_fetch_opts("data.table", ...)
+  opts <- list(...)$opts$data.table %||% opts_data.table()
   if (is_corrupted_data.table(x) || opts$constructor == "next") return(NextMethod())
-  constructor <- constructors$data.table[[opts$constructor]]
-  constructor(x, selfref = opts$selfref, ...)
+  UseMethod(".cstr_construct.data.table", structure(NA, class = opts$constructor))
 }
 
 is_corrupted_data.table <- function(x) {
   is_corrupted_data.frame(x)
 }
 
-constructors$data.table$list <- function(x, ...) {
+#' @export
+#' @method .cstr_construct.data.table list
+.cstr_construct.data.table.list <- function(x, ...) {
   .cstr_construct.list(x, ...)
 }
 
-constructors$data.table$data.table <- function(x, selfref, ...) {
+#' @export
+#' @method .cstr_construct.data.table data.table
+.cstr_construct.data.table.data.table <- function(x, ...) {
+  opts <- list(...)$opts$data.table %||% opts_data.table()
+  # Fall back on list constructor if relevant
+  arg_names <- c("keep.rownames", "check.names", "key", "stringsAsFactors")
+  df_has_problematic_names <- any(names(x) %in% arg_names)
+  if (df_has_problematic_names) return(.cstr_construct.list(x, ...))
+
   key <- attr(x, "sorted")
   if (!is.null(key)) {
     args <- c(x, key = key)
@@ -52,7 +54,7 @@ constructors$data.table$data.table <- function(x, selfref, ...) {
     args <- x
   }
   code <- .cstr_apply(args, fun = "data.table::data.table", ...)
-  repair_attributes_data.table(x, code, ..., selfref = selfref)
+  repair_attributes_data.table(x, code, ..., selfref = opts$selfref)
 }
 
 repair_attributes_data.table <- function(x, code, ..., pipe = NULL, selfref = FALSE) {
