@@ -1,40 +1,36 @@
-constructors$externalptr <- new.env()
-
 #' Constructive options for type 'externalptr'
 #'
 #' These options will be used on objects of type 'externalptr'. By default this
 #' function is useless as nothing can be set, this is provided in case users wan
 #' to extend the method with other constructors.
 #'
-#' Depending on `constructor`, we construct the environment as follows:
+#' Depending on `constructor`, we construct the object as follows:
 #' * `"default"` : We use a special function from the constructive
 #'
-#' @param constructor String. Name of the function used to construct the environment.
+#' @param constructor String. Name of the function used to construct the object.
 #' @inheritParams opts_atomic
 #'
-#' @return An object of class <constructive_options/constructive_options_environment>
+#' @return An object of class <constructive_options/constructive_options_externalptr>
 #' @export
 opts_externalptr  <- function(constructor = c("default"), ...) {
-  .cstr_combine_errors(
-    constructor <- .cstr_match_constructor(constructor, "externalptr"),
-    check_dots_empty()
-  )
-  .cstr_options("externalptr", constructor = constructor)
+  .cstr_options("externalptr", constructor = constructor[[1]], ...)
 }
 
 #' @export
+#' @method .cstr_construct externalptr
 .cstr_construct.externalptr <- function(x, ...) {
-  opts <- .cstr_fetch_opts("externalptr", ...)
+  opts <- list(...)$opts$externalptr %||% opts_externalptr()
   if (is_corrupted_externalptr (x)) return(NextMethod())
-  constructor <- constructors$externalptr [[opts$constructor]]
-  constructor(x, ...)
+  UseMethod(".cstr_construct.externalptr", structure(NA, class = opts$constructor))
 }
 
 is_corrupted_externalptr <- function(x) {
   typeof(x) != "externalptr"
 }
 
-constructors$externalptr$default <- function(x, ...) {
+#' @export
+#' @method .cstr_construct.externalptr default
+.cstr_construct.externalptr.default <- function(x, ...) {
   code <- sprintf('constructive::.xptr("%s")', external_pointer_address(x))
   repair_attributes_externalptr(x, code, ...)
 }
