@@ -1,10 +1,8 @@
-constructors$`function` <- new.env()
-
 #' Constructive options for functions
 #'
 #' These options will be used on functions, i.e. objects of type "closure", "special" and "builtin".
 #'
-#' Depending on `constructor`, we construct the environment as follows:
+#' Depending on `constructor`, we construct the object as follows:
 #' * `"function"` (default): Build the object using a standard `function() {}`
 #'   definition. This won't set the environment by default, unless `environment`
 #'   is set to `TRUE`. If a srcref is available, if this srcref matches the function's
@@ -16,7 +14,7 @@ constructors$`function` <- new.env()
 #'   back to `data.frame()`.
 #' * `"new_function"` : Build the object using a `rlang::new_function()` call.
 #'
-#' @param constructor String. Name of the function used to construct the environment, see Details section.
+#' @param constructor String. Name of the function used to construct the object, see Details section.
 #' @inheritParams opts_atomic
 #' @param environment Boolean. Whether to reconstruct the function's environment.
 #' @param srcref Boolean. Whether to attempt to reconstruct the function's srcref.
@@ -33,68 +31,55 @@ opts_function <- function(
     srcref = FALSE,
     trim = NULL) {
   .cstr_combine_errors(
-    constructor <- .cstr_match_constructor(constructor, "function"),
     check_dots_empty(),
     abort_not_boolean(environment),
     abort_not_boolean(srcref),
     abort_not_null_or_integerish(trim)
   )
-  .cstr_options("function", constructor = constructor, environment = environment, srcref = srcref, trim = trim)
+  .cstr_options("function", constructor = constructor[[1]], environment = environment, srcref = srcref, trim = trim)
 }
 
-
 #' @export
-.cstr_construct.function <- function(
-    x, ..., pipe, one_liner = FALSE) {
+#' @method .cstr_construct function
+.cstr_construct.function <- function(x, ...) {
   if (rlang::is_primitive(x)) return(deparse(x))
-  opts <- .cstr_fetch_opts("function", ...)
+  opts <- list(...)$opts$`function` %||% opts_function()
   if (is_corrupted_function(x)) return(NextMethod())
-
-  # trim if relevant
-  trim <- opts$trim
-  if (!is.null(trim)) {
-    x_lst <- as.list(unclass(x))
-    x_length <- length(x_lst)
-    body_lng <- x_lst[[x_length]]
-    if (length(body_lng) > trim + 1) {
-      x_lst[[x_length]] <- as.call(c(head(as.list(body_lng), trim + 1), quote(...)))
-      x <- as.function(x_lst, envir = environment(x))
-    }
-  }
-
-  constructor <- constructors$`function`[[opts$constructor]]
-  constructor(x, ..., trim = opts$trim, environment = opts$environment, srcref = opts$srcref)
+  UseMethod(".cstr_construct.function", structure(NA, class = opts$constructor))
 }
 
 is_corrupted_function <- function(x) {
   !is.function(x)
 }
 
-constructors$`function`$`function` <- function(x, ..., pipe = NULL, one_liner = FALSE, trim, environment, srcref) {
+#' @export
+#' @method .cstr_construct.function function
+.cstr_construct.function.function <- function(x, ...) {
+  opts <- list(...)$opts$`function` %||% opts_function()
+  trim <- opts$trim
+  environment <- opts$environment
+  srcref <- opts$srcref
+
+  x_bkp <- x
+  if (!is.null(trim)) x <- trim_function(x, trim)
+
   # if the srcref matches the function's body (always in non artifical cases)
   # we might use the srcref rather than the body, so we keep the comments
 
   x_lst <- as.list(unclass(x))
   x_length <- length(x_lst)
-  body_is_a_proper_expression <-
-    is_expression2(x_lst[[x_length]])
 
-  if (!body_is_a_proper_expression) {
+  all_components_are_proper_expressions <-
+    all(vapply(x_lst, is_expression2, logical(1)))
+
+  if (!all_components_are_proper_expressions) {
     # fall back on `as.function()` constructor
-    res <- constructors$`function`$as.function(
-      x,
-      ...,
-      pipe = pipe,
-      one_liner = one_liner,
-      trim = trim,
-      environment = environment,
-      srcref = srcref
-    )
+    res <- .cstr_construct.function.as.function(x, ...)
     return(res)
   }
 
   code_from_srcref <- FALSE
-  if (!one_liner && is.null(trim)) {
+  if (!list(...)$one_liner && is.null(trim)) {
     code <- code_from_srcref(x)
     if (!is.null(code)) {
       code_from_srcref <- TRUE
@@ -107,13 +92,7 @@ constructors$`function`$`function` <- function(x, ..., pipe = NULL, one_liner =
       fun_call[[2]] <- as.pairlist(x_lst[-x_length])
     }
     fun_call[3] <- x_lst[x_length]
-    code <- deparse_call(
-      fun_call,
-      pipe = FALSE,
-      one_liner = one_liner,
-      style = FALSE,
-      collapse = FALSE
-    )
+    code <- deparse_call0(fun_call, ...)
     if (length(code) == 2) code <- paste(code[1], code[2])
   }
 
@@ -124,34 +103,34 @@ constructors$`function`$`function` <- function(x, ..., pipe = NULL, one_liner =
     code <- .cstr_wrap(code, fun = "")
   }
   if (environment) {
-    envir_code <- .cstr_apply(
-      list(environment(x)),
-      "(`environment<-`)",
-      pipe = pipe,
-      one_liner = one_liner,
-      ...)
-    code <- .cstr_pipe(code, envir_code, pipe, one_liner)
+    envir_code <- .cstr_apply(list(environment(x)), "(`environment<-`)", ...)
+    code <- .cstr_pipe(code, envir_code, ...)
   }
-  repair_attributes_function(x, code, ..., pipe = pipe, one_liner = one_liner)
+  repair_attributes_function(x_bkp, code, ...)
 }
 
-constructors$`function`$as.function <- function(x, ..., trim, environment, srcref) {
-  # rlang::expr_deparse changes the body by putting parentheses around f <- (function(){})
-  # so we must use regular deparse
+#' @export
+#' @method .cstr_construct.function as.function
+.cstr_construct.function.as.function <- function(x, ...) {
+  opts <- list(...)$opts$`function` %||% opts_function()
+  trim <- opts$trim
+  environment <- opts$environment
+  srcref <- opts$srcref
+
+  x_bkp <- x
+  if (!is.null(trim)) x <- trim_function(x)
 
   x_lst <- as.list(unclass(x))
 
-  body_is_a_proper_expression <-
-    is_expression2(x_lst[[length(x_lst)]])
+  all_components_are_proper_expressions <-
+    all(vapply(x_lst, is_expression2, logical(1)))
 
-  if (body_is_a_proper_expression) {
-    fun_lst <- lapply(x_lst, deparse_call, style = FALSE, collapse = FALSE)
-    args <- list(.cstr_apply(
-      fun_lst, "alist", ..., recurse = FALSE))
+  if (all_components_are_proper_expressions) {
+    fun_lst <- lapply(x_lst, deparse_call0, ...)
+    args <- list(.cstr_apply(fun_lst, "alist", ..., recurse = FALSE))
   } else {
-    fun_lst <- lapply(x_lst, .cstr_construct.language, style = FALSE, collapse = FALSE)
-    args <- list(.cstr_apply(
-      fun_lst, "list", ..., recurse = FALSE))
+    fun_lst <- lapply(x_lst, function(x, ...) .cstr_construct(x, ...), ...)
+    args <- list(.cstr_apply(fun_lst, "list", ..., recurse = FALSE))
   }
 
   if (environment) {
@@ -159,15 +138,25 @@ constructors$`function`$as.function <- function(x, ..., trim, environment, srcre
     args <- c(args, list(envir = envir_arg))
   }
   code <- .cstr_apply(args, "as.function", ..., recurse = FALSE)
-  repair_attributes_function(x, code, ...)
+  repair_attributes_function(x_bkp, code, ...)
 }
 
-constructors$`function`$new_function <- function(x, ..., trim, environment, srcref) {
+#' @export
+#' @method .cstr_construct.function new_function
+.cstr_construct.function.new_function <- function(x, ...) {
+  opts <- list(...)$opts$`function` %||% opts_function()
+  trim <- opts$trim
+  environment <- opts$environment
+  srcref <- opts$srcref
+
+  x_bkp <- x
+  if (!is.null(trim)) x <- trim_function(x)
+
   x_lst <- as.list(unclass(x))
 
-  args <- lapply(x_lst[-length(x_lst)], deparse_call)
+  args <- lapply(x_lst[-length(x_lst)], deparse_call0, ...)
   args <- .cstr_apply(args, "alist", ..., recurse = FALSE)
-  body <- .cstr_construct.language(x_lst[[length(x_lst)]])
+  body <- .cstr_construct.language(x_lst[[length(x_lst)]], ...)
 
   args <- list(args = args, body = body)
   if (environment) {
@@ -175,19 +164,15 @@ constructors$`function`$new_function <- function(x, ..., trim, environment, srcr
     args <- c(args, list(env = envir_arg))
   }
   code <- .cstr_apply(args, "rlang::new_function", ..., recurse = FALSE)
-  repair_attributes_function(x, code, ...)
+  repair_attributes_function(x_bkp, code, ...)
 }
 
-repair_attributes_function <- function(x, code, ..., pipe = NULL) {
-  opts <- .cstr_fetch_opts("function", ...)
-  srcref <- opts$srcref
+repair_attributes_function <- function(x, code, ...) {
+  opts <- list(...)$opts$`function` %||% opts_function()
+  srcref <- opts[["srcref"]]
   ignore <- c("name", "path")
   if (!srcref) ignore <- c(ignore, "srcref")
-  .cstr_repair_attributes(
-    x, code, ...,
-    pipe = pipe,
-    ignore = ignore
-  )
+  .cstr_repair_attributes(x, code, opts = opts, ..., ignore = ignore)
 }
 
 # returns the srcref as a character vector IF it matches the actual function, NULL otherwise
@@ -209,3 +194,14 @@ code_from_srcref <- function(x) {
   }
   srcref_chr
 }
+
+trim_function <- function(x, trim) {
+  x_lst <- as.list(unclass(x))
+  x_length <- length(x_lst)
+  body_lng <- x_lst[[x_length]]
+  if (length(body_lng) > trim + 1) {
+    x_lst[[x_length]] <- as.call(c(head(as.list(body_lng), trim + 1), quote(...)))
+    x <- as.function(x_lst, envir = environment(x))
+  }
+  x
+}
