@@ -1,35 +1,29 @@
-constructors$quosure <- new.env()
-
 #' Constructive options for class 'quosure'
 #'
 #' These options will be used on objects of class 'quosure'.
 #'
-#' Depending on `constructor`, we construct the environment as follows:
+#' Depending on `constructor`, we construct the object as follows:
 #' * `"new_quosure"` (default): Build the object using a `new_quosure()` call on a
 #' character vector.
 #' * `"next"` : Use the constructor for the next supported class. Call `.class2()`
 #'   on the object to see in which order the methods will be tried.
 #' * `"language"` : We define as an language object and repair attributes.
 #'
-#' @param constructor String. Name of the function used to construct the environment, see Details section.
+#' @param constructor String. Name of the function used to construct the object, see Details section.
 #' @inheritParams opts_atomic
 #'
-#' @return An object of class <constructive_options/constructive_options_factor>
+#' @return An object of class <constructive_options/constructive_options_quosure>
 #' @export
 opts_quosure <- function(constructor = c("new_quosure", "next", "language"), ...) {
-  .cstr_combine_errors(
-    constructor <- .cstr_match_constructor(constructor, "quosure"),
-    check_dots_empty()
-  )
-  .cstr_options("quosure", constructor = constructor)
+  .cstr_options("quosure", constructor = constructor[[1]], ...)
 }
 
 #' @export
+#' @method .cstr_construct quosure
 .cstr_construct.quosure <- function(x, ...) {
-  opts <- .cstr_fetch_opts("quosure", ...)
+  opts <- list(...)$opts[["quosure"]] %||% opts_quosure()
   if (is_corrupted_quosure(x) || opts$constructor == "next") return(NextMethod())
-  constructor <- constructors$quosure[[opts$constructor]]
-  constructor(x, ..., origin = opts$origin)
+  UseMethod(".cstr_construct.quosure", structure(NA, class = opts$constructor))
 }
 
 is_corrupted_quosure <- function(x) {
@@ -38,7 +32,8 @@ is_corrupted_quosure <- function(x) {
 }
 
 #' @export
-constructors$quosure$new_quosure <- function(x, env, ...) {
+#' @method .cstr_construct.quosure new_quosure
+.cstr_construct.quosure.new_quosure <- function(x, env = NULL, ...) {
   if (identical(env, attr(x, ".Environment"))) {
     code <- .cstr_apply(list(rlang::quo_squash(x)), "rlang::new_quosure", ...)
   } else {
@@ -48,7 +43,8 @@ constructors$quosure$new_quosure <- function(x, env, ...) {
 }
 
 #' @export
-constructors$quosure$language <- function(x, ...) {
+#' @method .cstr_construct.quosure language
+.cstr_construct.quosure.language <- function(x, ...) {
   .cstr_construct.language(x, ...)
 }
 
