@@ -1,35 +1,29 @@
-constructors$quosures <- new.env()
-
 #' Constructive options for class 'quosures'
 #'
 #' These options will be used on objects of class 'quosures'.
 #'
-#' Depending on `constructor`, we construct the environment as follows:
+#' Depending on `constructor`, we construct the object as follows:
 #' * `"as_quosures"` (default): Build the object using a `as_quosures()` call on a
 #' character vector.
 #' * `"next"` : Use the constructor for the next supported class. Call `.class2()`
 #'   on the object to see in which order the methods will be tried.
 #' * `"list"` : We define as an list object and repair attributes.
 #'
-#' @param constructor String. Name of the function used to construct the environment, see Details section.
+#' @param constructor String. Name of the function used to construct the object, see Details section.
 #' @inheritParams opts_atomic
 #'
-#' @return An object of class <constructive_options/constructive_options_factor>
+#' @return An object of class <constructive_options/constructive_options_quosures>
 #' @export
 opts_quosures <- function(constructor = c("new_quosures", "next", "list"), ...) {
-  .cstr_combine_errors(
-    constructor <- .cstr_match_constructor(constructor, "quosures"),
-    check_dots_empty()
-  )
-  .cstr_options("quosures", constructor = constructor)
+  .cstr_options("quosures", constructor = constructor[[1]], ...)
 }
 
 #' @export
+#' @method .cstr_construct quosures
 .cstr_construct.quosures <- function(x, ...) {
-  opts <- .cstr_fetch_opts("quosures", ...)
+  opts <- list(...)$opts$quosures %||% opts_quosures()
   if (is_corrupted_quosures(x) || opts$constructor == "next") return(NextMethod())
-  constructor <- constructors$quosures[[opts$constructor]]
-  constructor(x, ...)
+  UseMethod(".cstr_construct.quosures", structure(NA, class = opts$constructor))
 }
 
 is_corrupted_quosures <- function(x) {
@@ -38,7 +32,8 @@ is_corrupted_quosures <- function(x) {
 }
 
 #' @export
-constructors$quosures$new_quosures <- function(x, ...) {
+#' @method .cstr_construct.quosures new_quosures
+.cstr_construct.quosures.new_quosures <- function(x, ...) {
   x_list <- unclass(x)
   # remove names if "" so we avoid repairing the names
   if (all(names(x) == "")) names(x_list) <- NULL
@@ -48,7 +43,8 @@ constructors$quosures$new_quosures <- function(x, ...) {
 }
 
 #' @export
-constructors$quosures$list <- function(x, ...) {
+#' @method .cstr_construct.quosures list
+.cstr_construct.quosures.list <- function(x, ...) {
   .cstr_construct.list(x, ...)
 }
 
