@@ -1,5 +1,5 @@
 test_that("structure corner cases", {
   a <- 1
   attributes(a) <- list(.Names = "name1", names = "name2", .Label = "label1", levels = "label2")
-  expect_pipe_snapshot(construct(a))
+  expect_snapshot(construct(a))
 })
