@@ -1,5 +1,5 @@
 test_that("grouped_df", {
-  expect_pipe_snapshot({
+  expect_snapshot({
     construct(dplyr::group_by(head(cars,2), dist))
   })
 })
