@@ -7,6 +7,16 @@ test_that("language", {
   })
 })
 
+test_that("language after 4.1", {
+  # Due to bypass.R
+  skip_if(base::`<`(getRversion(), "4.1"))
+
+  expect_snapshot({
+    construct(quote(`🐶`))
+    construct(quote(`🐶`), unicode_representation = "unicode")
+  })
+})
+
 test_that("complex language", {
   expect_snapshot({
     x <- quote(a(1)(2))
@@ -18,4 +28,3 @@ test_that("complex language", {
     construct(y)
   })
 })
-
